// This is an amalgation of the jasper library by mvh 20100117

#include "include/jasper/jasper.h"

#include "base/jas_cm.c"
#include "base/jas_debug.c"
#include "base/jas_getopt.c"
#include "base/jas_icc.c"
#include "base/jas_iccdata.c"
#include "base/jas_image.c"
#include "base/jas_init.c"
#include "base/jas_malloc.c"
#include "base/jas_seq.c"
#include "base/jas_stream.c"
#include "base/jas_string.c"
#include "base/jas_tmr.c"
#include "base/jas_tvp.c"
#include "base/jas_version.c"

#include "bmp/bmp_cod.h"
#include "bmp/bmp_enc.h"
#include "bmp/bmp_cod.c"
#include "bmp/bmp_dec.c"
#include "bmp/bmp_enc.c"

#include "jp2/jp2_cod.h"
#include "jp2/jp2_dec.h"
#include "jp2/jp2_cod.c"
#include "jp2/jp2_dec.c"
#include "jp2/jp2_enc.c"

#include "jpc/jpc_bs.h"
#include "jpc/jpc_cod.h"
#include "jpc/jpc_cs.h"
#include "jpc/jpc_dec.h"
#include "jpc/jpc_enc.h"
#include "jpc/jpc_fix.h"
#include "jpc/jpc_flt.h"
#include "jpc/jpc_math.h"
#include "jpc/jpc_mct.h"
#include "jpc/jpc_mqcod.h"
#include "jpc/jpc_mqdec.h"
#include "jpc/jpc_mqenc.h"
#include "jpc/jpc_qmfb.h"
#include "jpc/jpc_t1cod.h"
#include "jpc/jpc_t1dec.h"
#include "jpc/jpc_t1enc.h"
#include "jpc/jpc_t2cod.h"
#include "jpc/jpc_t2dec.h"
#include "jpc/jpc_t2enc.h"
#include "jpc/jpc_tagtree.h"
#include "jpc/jpc_tsfb.h"
#include "jpc/jpc_util.h"

#include "jpc/jpc_bs.c"
#include "jpc/jpc_cs.c"
#include "jpc/jpc_dec.c"    // had to change OPT_DEBUG to OPTx_DEBUG because of name clash
#include "jpc/jpc_enc.c"
#include "jpc/jpc_math.c"
#include "jpc/jpc_mct.c"
#include "jpc/jpc_mqcod.c"
#include "jpc/jpc_mqdec.c"
#include "jpc/jpc_mqenc.c"
#include "jpc/jpc_qmfb.c"
#include "jpc/jpc_t1cod.c"
#include "jpc/jpc_t1dec.c"
#include "jpc/jpc_t1enc.c"
#include "jpc/jpc_t2cod.c"
#include "jpc/jpc_t2dec.c"
#include "jpc/jpc_t2enc.c"
#include "jpc/jpc_tagtree.c"
#include "jpc/jpc_tsfb.c"
#include "jpc/jpc_util.c"

#include "jpg/jpg_cod.h"
#include "jpg/jpg_enc.h"
//#include "jpg/jpg_jpeglib.h"

//#include "jpg/jpg_dec.c"
//#include "jpg/jpg_enc.c"
#include "jpg/jpg_dummy.c"
#include "jpg/jpg_val.c"

#include "mif/mif_cod.h"
#include "mif/mif_cod.c"

#include "pgx/pgx_cod.h"
#include "pgx/pgx_enc.h"
#include "pgx/pgx_cod.c"
#include "pgx/pgx_dec.c"
#include "pgx/pgx_enc.c"

#include "pnm/pnm_cod.h"
#include "pnm/pnm_enc.h"
#include "pnm/pnm_cod.c"
#include "pnm/pnm_dec.c"
#include "pnm/pnm_enc.c"

#include "ras/ras_cod.h"
#include "ras/ras_enc.h"
#include "ras/ras_cod.c"
#include "ras/ras_dec.c"
#include "ras/ras_enc.c"

