-- 1.4.17d
--[[ These are sample scripts that can be run by [Conquest DICOM server](http://forum.image-systems.biz/viewforum.php?f=33).
You need to use version 1.4.17 or later for most samples to work
 
- Switch to a standalone Conquest interpreter by going to `Project | Lua Interpreter | Conquest DICOM Utility`.
- Connect to a running Conquest server by going to `Project | Lua Interpreter | Conquest DICOM Server`.

- Open one of the following scripts:
  - [livecoding](+/livecoding.lua): a simple live coding script
    - Run this script using `Project | Run as Scratchpad`.
--]]
